module Api exposing (addUser, fetchLesson, fetchLessons)

import Dict exposing (Dict)
import Http
import Json.Decode as Decode
import Json.Encode as Encode
import Types exposing (Card, Lesson, Lessons, Msg(..), ServerResponse(..))



-- data types
-- json decoders


serverResponseDecoder : Decode.Decoder t -> Decode.Decoder (ServerResponse t)
serverResponseDecoder okDecoder =
    Decode.oneOf
        [ Decode.map OkResponse
            (Decode.field "ok" okDecoder)
        , Decode.map ErrorResponse (Decode.field "error" Decode.string)
        ]


cardDecoder : Decode.Decoder Card
cardDecoder =
    Decode.map3 Card
        (Decode.field "text" Decode.string)
        (Decode.field "note" (Decode.maybe Decode.string))
        (Decode.field "id" Decode.int)


lessonDecoder : Decode.Decoder Lesson
lessonDecoder =
    Decode.map3 Lesson
        (Decode.field "text" Decode.string)
        (Decode.field "id" Decode.int)
        (Decode.field "cards" (Decode.list cardDecoder))


lessonsDecoder : Decode.Decoder Lessons
lessonsDecoder =
    Decode.dict lessonDecoder
        |> Decode.andThen convertKeysToIntDict


convertKeysToIntDict : Dict String Lesson -> Decode.Decoder Lessons
convertKeysToIntDict stringKeyedDict =
    stringKeyedDict
        |> Dict.toList
        |> List.map (\( key, lesson ) -> ( String.toInt key, lesson ))
        |> List.foldl
            (\maybePair acc ->
                case maybePair of
                    ( Just intKey, lesson ) ->
                        Dict.insert intKey lesson acc

                    ( Nothing, _ ) ->
                        acc
            )
            Dict.empty
        |> Decode.succeed



-- json encoders


newUserEncoder name creds =
    Encode.object [ ( "name", Encode.string name ), ( "creds", Encode.string creds ) ]



-- http command


fetchLessons : Cmd Msg
fetchLessons =
    Http.get
        { url = "http://localhost:3000/api/lessons"
        , expect = Http.expectJson FetchLessons (serverResponseDecoder lessonsDecoder)
        }


fetchLesson : Int -> Cmd Msg
fetchLesson num =
    Http.get
        { url = "http://localhost:3000/api/lesson/" ++ String.fromInt num
        , expect = Http.expectJson FetchLesson (serverResponseDecoder lessonDecoder)
        }



-- post requests


addUser : String -> String -> Cmd Msg
addUser name pw =
    Http.post
        { url = "http://localhost:3000/api/lessons"
        , body = Http.jsonBody (newUserEncoder name pw)
        , expect = Http.expectJson FetchLessons (serverResponseDecoder lessonsDecoder)
        }



-- FetchRawJsonHandler (Ok rawJson) ->
--     Debug.log "Raw json response" rawJson
--         |> (\_ ->
--                 case Decode.decodeString lessonsDecoder rawJson of
--                     Ok lessons ->
--                         ( { model | lessons = lessons, isLoading = False }, Cmd.none )
--                     Err decodeError ->
--                         Debug.log "Decode Error" decodeError
--                             |> (\_ -> ( { model | isLoading = False }, Cmd.none ))
--            )
-- FetchRawJsonHandler (Err httpError) ->
--     Debug.log "HTTP Error" httpError
--         |> (\_ -> ( { model | isLoading = False }, Cmd.none ))
